@@ -55,7 +55,8 @@ sub response_cb {
                         $line = $filter_cb->($line);
                     }
                     # Send EOF.
-                    push @{ $res->[2] }, $filter_cb->( undef );
+                    my $eof = $filter_cb->( undef );
+                    push @{ $res->[2] }, $eof if defined $eof;
                 } else {
                     my $body    = $res->[2];
                     my $getline = sub { $body->getline };
@@ -75,7 +76,7 @@ sub response_cb {
     } elsif (ref $res eq 'CODE') {
         return sub {
             my $respond = shift;
-            my $cb = $cb;  # To avoid the nested closure leek for 5.8.x
+            my $cb = $cb;  # To avoid the nested closure leak for 5.8.x
             $res->(sub {
                 my $res = shift;
                 my $filter_cb = $body_filter->($cb, $res);
